#include <Wire.h>
#include <U8g2lib.h>

// Create an OLED display object (U8G2_SSD1306_128X64_NONAME_F_HW_I2C for hardware I2C)
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE, 16, 17); // SCL=17, SDA=16

// Define the pins connected to the encoder
#define ENCODER_PIN_A 3
#define ENCODER_PIN_B 2
#define ENCODER_BUTTON_PIN 19

// Variables to store encoder state
volatile int encoderPosition = 0;
volatile bool A_set = false;
volatile bool B_set = false;
bool up = false;

void setup() {
  // Initialize serial communication for debugging
  Serial.begin(9600);

  // Initialize the OLED display
  u8g2.begin();
  u8g2.clearBuffer();

  // Set encoder pins as inputs
  pinMode(ENCODER_PIN_A, INPUT);
  pinMode(ENCODER_PIN_B, INPUT);
  pinMode(ENCODER_BUTTON_PIN, INPUT_PULLUP);

  // Attach interrupts to the encoder pins
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_A), isr0, FALLING);
  attachInterrupt(digitalPinToInterrupt(ENCODER_BUTTON_PIN), button, CHANGE);
}

void loop() {
  // Print the encoder position to the serial monitor
  Serial.print("Encoder Position: ");
  Serial.println(encoderPosition);

  // Display the encoder position on the OLED
  // u8g2.clearBuffer();
  // u8g2.setFont(u8g2_font_ncenB14_tr);
  // u8g2.setCursor(0, 24);
  // u8g2.print("Pos: ");
  // u8g2.setCursor(0, 48);
  // u8g2.print(encoderPosition);
  // u8g2.sendBuffer();

  delay(10);
}

void isr0() {
  // Read the state of both encoder pins
  up = (digitalRead(ENCODER_PIN_A) == digitalRead(ENCODER_PIN_B));
  if (up) {
    encoderPosition++;
  } else {
    encoderPosition--;
  }

  oledUpdate ();
}

void button() {
  Serial.println("Button pressed");
}

void oledUpdate ()
{
    // Display the encoder position on the OLED
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_ncenB14_tr);
  u8g2.setCursor(0, 24);
  u8g2.print("Pos: ");
  u8g2.setCursor(0, 48);
  u8g2.print(encoderPosition);
  u8g2.sendBuffer();
}
