#define SERVO_PIN 9
#define SHOOTER_MOTOR_PIN 11
#define ENCODER_A 22
#define ENCODER_B 18
#define ENCODER_SW 19

// Stepper pin Config
#define STEPPER_DIR 8
#define STEPPER_EN 3
#define STEPPER_STEP 10
#define STEPPER_VREF 2

#define DISABLED 1
#define ENABLED 0

// Microstepping setting for DRV8825
#define MICROSTEPPING 32  // 1/32 microstepping

// Gear ratio (input rotations : output rotations)
#define GEAR_RATIO 11  // 5:1 gear ratio

#define STEP_PER_REVOLUTION 48  // motors steps needed for 1 full revolution

#include <AccelStepper.h>
#include <Servo.h>
#include <Wire.h>
#include <U8g2lib.h>

U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE, 16, 17); // SCL=17, SDA=16


Servo myservo;

long stepPerRevolution = STEP_PER_REVOLUTION * MICROSTEPPING * GEAR_RATIO;
long stepPerDegree = stepPerRevolution / 360;

AccelStepper stepper = AccelStepper(1, STEPPER_STEP, STEPPER_DIR);

void setup() {
  Serial.begin(115200);
  pinMode(STEPPER_EN, OUTPUT);
  pinMode(STEPPER_VREF, OUTPUT);
  digitalWrite(STEPPER_VREF, HIGH);
  pinMode(SHOOTER_MOTOR_PIN, OUTPUT);  // Set shooter motor pin as output
  myservo.attach(SERVO_PIN);           // Attach the servo on pin 9 to the Servo object
  myservo.write(90);  // Stop
  delay(15);

    u8g2.begin();
  u8g2.clearBuffer();


  // while (!Serial) {
  //   ;  // Wait for serial port to connect. Needed for native USB port only
  // }
  // Serial.println("Enter the number to divide 360 by:");

  digitalWrite(STEPPER_EN, ENABLED);

  stepper.setMaxSpeed(4000);      // steps per second (full steps)
  stepper.setAcceleration(3000);  // steps per second squared (full steps)

displayWelcomeMessage();
  dispence(3,5); // enter how many players as first (x), and how many cards as second(y)

displayFinishedMessage();
}

void loop() {

}

// Function to shuffle an array using Fisher-Yates algorithm
void shuffleArray(int array[], int n) {
  for (int i = n - 1; i > 0; i--) {
    int j = random(0, i + 1);  // Generate a random number between 0 and i
    // Swap array[i] with the element at random index
    int temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }
}

void moveStepper(long position) {
  position = position * stepPerDegree;

  // Move the motor to the target position
  stepper.moveTo(position);

  // Run the motor to the target position
  while (stepper.distanceToGo() != 0) {
    stepper.run();
  }
  // stepper.setCurrentPosition(0);
}


void dispence(int x, int y)
{
  if (x > 0 && y > 0) {
      for (int j = 0; j < y; j++) {
        Serial.print("Cycle: ");
        Serial.println(j + 1);
        // Start the shooter motor
        // Create an array to hold the angles
        int angles[x];

        // Populate the array with divided angles
        for (int i = 0; i < x; i++) {
          angles[i] = (360 / x) * i;
        }

        // Shuffle the array
        shuffleArray(angles, x);

        // Print the shuffled angles
        Serial.println("Shuffled angles:");
        stepper.setCurrentPosition(0);
        digitalWrite(SHOOTER_MOTOR_PIN, HIGH);

        for (int i = 0; i < x; i++) {
          Serial.println(angles[i]);

          // Rotate the servo clockwise
          myservo.write(180);  // Full speed clockwise
          delay(400);          // Rotate for a moment

          // Stop the servo
          myservo.write(90);  // Stop
          delay(1000);        // Pause for a moment
          moveStepper(angles[i]);
        }
        // Stop the shooter motor
        digitalWrite(SHOOTER_MOTOR_PIN, LOW);
      }
    } 
}


void displayWelcomeMessage() {
  // Serial.println("");

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setCursor(0, 16);
  u8g2.print("Serving cards");
  u8g2.sendBuffer();
}

void displayFinishedMessage() {
  // Serial.println("");

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setCursor(0, 16);
  u8g2.print("Serve completed.");
  u8g2.sendBuffer();
}



