#include <Wire.h>
#include <U8g2lib.h>

// Create an OLED display object (U8G2_SSD1306_128X64_NONAME_F_HW_I2C for hardware I2C)
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE, 16, 17); // SCL=17, SDA=16

// Define the pins connected to the encoder
#define ENCODER_PIN_A 22
#define ENCODER_PIN_B 18
#define ENCODER_BUTTON_PIN 19

// Variables to store encoder state
volatile int encoderPosition = 0;
volatile bool A_set = false;
volatile bool B_set = false;
bool up = false;

// Menu states
enum MenuState {WELCOME, SET_PERSONS, CONFIRM, SERVING};
MenuState menuState = WELCOME;

void setup() {
  // Initialize serial communication for debugging
  Serial.begin(9600);

  // Initialize the OLED display
  u8g2.begin();
  u8g2.clearBuffer();

  // Set encoder pins as inputs
  pinMode(ENCODER_PIN_A, INPUT);
  pinMode(ENCODER_PIN_B, INPUT);
  pinMode(ENCODER_BUTTON_PIN, INPUT_PULLUP);

  // Attach interrupts to the encoder pins
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_A), isr0, FALLING);
  attachInterrupt(digitalPinToInterrupt(ENCODER_BUTTON_PIN), button, FALLING);

  // Display the welcome message
  displayWelcomeMessage();
}

void loop() {
  // Main loop does nothing in this example
}

void isr0() {
  // Read the state of both encoder pins
  up = (digitalRead(ENCODER_PIN_A) == digitalRead(ENCODER_PIN_B));
  if (up) {
    encoderPosition++;
  } else {
    encoderPosition--;
  }

  Serial.print("Encoder Position: ");
  Serial.println(encoderPosition);

  if (menuState == SET_PERSONS) {
    oledUpdate();
  }
}

void button() {
  if (digitalRead(ENCODER_BUTTON_PIN) == LOW) {
    Serial.println("Button pressed");

    if (menuState == WELCOME) {
      Serial.println("State: WELCOME -> SET_PERSONS");
      menuState = SET_PERSONS;
      encoderPosition = 1;  // Initialize the number of persons
      oledUpdate();
    } else if (menuState == SET_PERSONS) {
      Serial.println("State: SET_PERSONS -> CONFIRM");
      menuState = CONFIRM;
      displayConfirmMessage();
    } else if (menuState == CONFIRM) {
      Serial.println("State: CONFIRM -> SERVING");
      menuState = SERVING;
      displayServingMessage();
    }
  }
}

void displayWelcomeMessage() {
  Serial.println("Displaying Welcome Message");

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setCursor(0, 16);
  u8g2.print("Welcome");
  u8g2.setCursor(0, 32);
  u8g2.print("Press knob to continue");
  u8g2.sendBuffer();
}

void oledUpdate() {
  if (menuState == SET_PERSONS) {
    // Display the number of persons
    Serial.print("Displaying Number of Persons: ");
    Serial.println(encoderPosition);

    u8g2.clearBuffer();
    u8g2.setFont(u8g2_font_6x10_tf);
    u8g2.setCursor(0, 16);
    u8g2.print("No of persons: ");
    u8g2.setCursor(0, 32);
    u8g2.print(encoderPosition);
    u8g2.sendBuffer();
  }
}

void displayConfirmMessage() {
  Serial.print("Displaying Confirmation Message: Persons = ");
  Serial.println(encoderPosition);

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setCursor(0, 16);
  u8g2.print("Persons: ");
  u8g2.setCursor(0, 32);
  u8g2.print(encoderPosition);
  u8g2.setCursor(0, 48);
  u8g2.print("Press knob to confirm");
  u8g2.sendBuffer();
}

void displayServingMessage() {
  Serial.print("Displaying Serving Message: Serving for ");
  Serial.println(encoderPosition);

  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setCursor(0, 16);
  u8g2.print("Serving cards for: ");
  u8g2.setCursor(0, 32);
  u8g2.print(encoderPosition);
  u8g2.sendBuffer();
}
