#include <Wire.h>
#include <U8g2lib.h>

// Function declarations
void welcomeScreen();
void oledUpdateWelcome();
void oledUpdateSelection();
void oledUpdateServing();
void isr();

// Create an OLED display object (U8G2_SSD1306_128X64_NONAME_F_HW_I2C for hardware I2C)
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE, 16, 17); // SCL=17, SDA=16

// Define the pins connected to the encoder
#define ENCODER_PIN_A 22
#define ENCODER_PIN_B 18
#define ENCODER_BUTTON_PIN 19

// Variables to store encoder state
volatile int encoderPosition = 0;
bool serving = false;
int numberOfPersons = 0;

void setup() {
  // Initialize serial communication for debugging
  Serial.begin(9600);

  // Initialize the OLED display
  u8g2.begin();
  u8g2.clearBuffer();

  // Set encoder pins as inputs
  pinMode(ENCODER_PIN_A, INPUT_PULLUP);
  pinMode(ENCODER_PIN_B, INPUT_PULLUP);
  pinMode(ENCODER_BUTTON_PIN, INPUT_PULLUP);

  // Attach interrupts to the encoder pins
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_A), isr, CHANGE);
  attachInterrupt(digitalPinToInterrupt(ENCODER_PIN_B), isr, CHANGE);

  // Initial display update
  welcomeScreen();
}

void loop() {
  // Nothing to do here as display is updated through interrupts and button press
}

void isr() {
  // Read the state of both encoder pins
  bool A_set = digitalRead(ENCODER_PIN_A);
  bool B_set = digitalRead(ENCODER_PIN_B);

  if (A_set == B_set) {
    encoderPosition--;
  } else {
    encoderPosition++;
  }

  // Adjust encoder position bounds
  if (encoderPosition < 0) {
    encoderPosition = 0;
  }

  // Update OLED display with number of persons
  if (!serving) {
    numberOfPersons = encoderPosition; // Update the number of persons directly based on encoder position
    oledUpdateSelection();
    Serial.print("Number of Persons: ");
    Serial.println(numberOfPersons);
  }
}

void welcomeScreen() {
  // Display welcome message
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_ncenB08_tr);
  u8g2.setCursor(0, 10);
  u8g2.print("Welcome!");
  u8g2.sendBuffer();
  Serial.println("Welcome!");

  // Wait for knob press to continue
  while (digitalRead(ENCODER_BUTTON_PIN) == HIGH);
  delay(500); // Debounce delay

  // Clear screen and display instructions
  u8g2.clearBuffer();
  u8g2.setCursor(0, 10);
  u8g2.print("Press the knob to continue");
  u8g2.sendBuffer();
  Serial.println("Press the knob to continue");
}

void oledUpdateSelection() {
  // Update OLED display with number of persons
  u8g2.clearBuffer();
  u8g2.setFont(u8g2_font_ncenB08_tr);
  u8g2.setCursor(0, 10);
  u8g2.print("Number of Persons: ");
  u8g2.print(numberOfPersons);
  u8g2.setCursor(0, 30);
  u8g2.print("Press knob to continue");
  u8g2.sendBuffer();
}
