
#define SERVO_PIN 9

#define SHOOTER_MOTOR_PIN 11

#define ENCODER_A 22
#define ENCODER_B 18
#define ENCODER_SW 19

// Stepper pin Config
#define STEPPER_DIR 8
#define STEPPER_EN 3
#define STEPPER_STEP 10
#define STEPPER_VREF 2

#define DISABLED 1
#define ENABLED 0

// Microstepping setting for DRV8825
#define MICROSTEPPING 32  // 1/32 microstepping

// Gear ratio (input rotations : output rotations)
#define GEAR_RATIO 11  // 5:1 gear ratio

#define STEP_PER_REVOLUTION 48 // motors steps needed for 1 full revolution

#include <AccelStepper.h>

long stepPerRevolution = STEP_PER_REVOLUTION * MICROSTEPPING * GEAR_RATIO;

long stepPerDegree = stepPerRevolution/360 ;

AccelStepper stepper = AccelStepper(1, STEPPER_STEP, STEPPER_DIR);


void setup() {
  Serial.begin(115200);
  pinMode(STEPPER_EN, OUTPUT);
  pinMode(STEPPER_VREF, OUTPUT);
  digitalWrite(STEPPER_VREF, HIGH);

    while (!Serial) {
    ; // Wait for serial port to connect. Needed for native USB port only
  }
  Serial.println("Enter the number to divide 360 by:");


  digitalWrite(STEPPER_EN, ENABLED);

  stepper.setMaxSpeed(4000);     // steps per second (full steps)
  stepper.setAcceleration(3000);  // steps per second squared (full steps)
}

void loop() {
// Check if data is available to read
  if (Serial.available() > 0) {
    // Read the input number
    int x = Serial.parseInt();

    // Check if the input number is valid
    if (x > 0) {
      // Create an array to hold the angles
      int angles[x];

      // Populate the array with divided angles
      for (int i = 0; i < x; i++) {
        angles[i] = (360 / x) * i;
      }

      // Shuffle the array
      shuffleArray(angles, x);

      // Print the shuffled angles
      Serial.println("Shuffled angles:");
      stepper.setCurrentPosition(0);
      for (int i = 0; i < x; i++) {
        Serial.println(angles[i]);
        moveStepper(angles[i]);
      }
    } else {
      Serial.println("Please enter a valid number greater than 0.");
    }

    // Clear the input buffer
    while (Serial.available() > 0) {
      Serial.read();
    }

    // Prompt for next input
    Serial.println("Enter the number to divide 360 by:");
  }
}


// Function to shuffle an array using Fisher-Yates algorithm
void shuffleArray(int array[], int n) {
  for (int i = n - 1; i > 0; i--) {
    int j = random(0, i + 1); // Generate a random number between 0 and i
    // Swap array[i] with the element at random index
    int temp = array[i];
    array[i] = array[j];
    array[j] = temp;
  }
}

void moveStepper(long position){

  position = position * stepPerDegree;
  
  // Move the motor to the target position
  stepper.moveTo(position);  

  // Run the motor to the target position
  while (stepper.distanceToGo() != 0) {
    stepper.run();
  }
  // stepper.setCurrentPosition(0);

}
