#include <U8g2lib.h>

// Define the OLED display width and height
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64

// Define I2C pins for the SAMD21
#define SDA_PIN 16
#define SCL_PIN 17

// Create an instance of the U8g2 display
U8G2_SSD1306_128X64_NONAME_F_SW_I2C display(U8G2_R0, /* clock=*/ SCL_PIN, /* data=*/ SDA_PIN, /* reset=*/ U8X8_PIN_NONE);

// Define pins for the rotary encoder and switch
#define CLK_PIN 18 // B pin of the encoder
#define DT_PIN 22  // A pin of the encoder
#define SW_PIN 19  // Pin connected to the switch

int counter = 0;  // Counter to keep track of encoder position
int currentStateCLK;
int lastStateCLK;
int currentStateSW;
int lastStateSW = HIGH; // Assume switch is not pressed initially
unsigned long lastDebounceTime = 0;  // The last time the switch input pin was toggled
unsigned long debounceDelay = 100;    // The debounce time in milliseconds

void setup() {
  // Set encoder pins as inputs
  pinMode(CLK_PIN, INPUT);
  pinMode(DT_PIN, INPUT);
  pinMode(SW_PIN, INPUT_PULLUP);  // Enable internal pull-up resistor for the switch

  // Initialize serial communication
  Serial.begin(9600);

  // Initialize the display
  display.begin();
  display.clearBuffer(); // Clear the display buffer
  display.setFont(u8g2_font_ncenB14_tr); // Set font
  display.setCursor(0, 20); // Set cursor position
  display.print("Hello, World!"); // Print text
  display.sendBuffer(); // Display the buffer content

  // Read the initial state of CLK
  lastStateCLK = digitalRead(CLK_PIN);
}

void loop() {
  // Read the current state of CLK
  currentStateCLK = digitalRead(CLK_PIN);

  // If the previous and current state of CLK are different, then a pulse occurred
  if (currentStateCLK != lastStateCLK && currentStateCLK == HIGH) {
    // Read the DT state
    int currentStateDT = digitalRead(DT_PIN);

    // If the DT state is different to the CLK state then the encoder is rotating counterclockwise
    if (currentStateDT != currentStateCLK) {
      counter--;
    } else {
      // Encoder is rotating clockwise
      counter++;
    }

    // Print the current counter value to the serial monitor
    Serial.print("Counter: ");
    Serial.println(counter);

    // Display the updated counter value on the OLED
    display.clearBuffer(); // Clear the display buffer
    display.setCursor(0, 0); // Set cursor position
    display.print("Counter: ");
    display.println(counter);
    display.sendBuffer(); // Display the buffer content
  }

  // Update lastStateCLK with the current state
  lastStateCLK = currentStateCLK;

  // Read the current state of the button
  int reading = digitalRead(SW_PIN);

  // Check if the button state has changed (with debounce)
  if (reading != lastStateSW) {
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay) {
    // If the button state has been stable for debounceDelay milliseconds
    if (reading != currentStateSW) {
      currentStateSW = reading;

      if (currentStateSW == LOW) {
        // Button is pressed
        Serial.println("Button pressed");
        // Update OLED display when button is pressed
        display.clearBuffer(); // Clear the display buffer
        display.setCursor(0, 0); // Set cursor position
        display.print("Button: Pressed");
        display.sendBuffer(); // Display the buffer content
      } else {
        // Button is released
        Serial.println("Button released");
        // Update OLED display when button is released
        display.clearBuffer(); // Clear the display buffer
        display.setCursor(0, 0); // Set cursor position
        display.print("Button: Released");
        display.sendBuffer(); // Display the buffer content
      }
    }
  }

  // Update the last button state
  lastStateSW = reading;
}
